//****************************************************************************//
//                                                                            //
//                                TXYPlot                                     //
//               A Borland C++ Builder Plotting Component                     //
//               ----------------------------------------                     //
//               TXYPlot is copyright 1998 Jeff Nicholson                     //
//                             jeffn@rt66.com                                 //
//                                                                            //
//   This software is provided "as is" without express or implied warranty.   //
//****************************************************************************//
#ifndef xyplotH
#define xyplotH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
#include <windef.h>
#include <wingdi.h>
//------------------------------------------------------------------------
// definition of axis style
enum TAxisStyle {Box, Centered, CenteredBox, LB, None};
//---------------------------------------------------------------------------
enum TAxisFormat {Linear, Log};
//---------------------------------------------------------------------------
// definition of plot style
enum TPlotStyle {Lines, FilledPoints, OpenPoints, LinesFilledPoints, LinesOpenPoints};
//---------------------------------------------------------------------------

class TXYPlot;
//---------------------------------------------------------------------------
// Class definition for a member of a linked list of Plots
class TPlotListMember
{
public:
   __fastcall TPlotListMember(void){Initialize();};
   __fastcall TPlotListMember(const TPlotListMember* PlotListMember);
   __fastcall ~TPlotListMember(void){CleanUp();};
   void Initialize(void);
   void CleanUp(void);
   double* x;      // x data
   double* y;      // y data
   TColor Color;    // color of plot
   TPlotStyle PlotStyle;  // style of plot
   int NumPoints;   // number of plot points
   int Index;       // index of the plot
private:
   TPlotListMember* NextPtr; // pointer to next plot member

   friend class TPlotList;
friend class TXYPlot;
};
//------------------------------------------------------------------------
// Class definition for the linked list of Plots
class TPlotList
{
public:
   __fastcall TPlotList(void){Initialize();};
   __fastcall TPlotList(const TPlotList* PlotList);
   __fastcall ~TPlotList(void){CleanUp();};
   void Initialize(void);
   void CleanUp(void);
   void AppendPlotList(const TPlotList* PlotList);
   bool AddPlot(double xin_array[], double yin_array[],int NumPoints, int PNumber,
                          TColor PColor, TPlotStyle PStyle);
   bool RemovePlot(int PNumber);
   bool FindPlot(TPlotListMember* &PlotPtr, int PNumber);
   TPlotListMember* FirstPtr;  // pointer to first plot member
};
//------------------------------------------------------------------------
class TPaintPage
{
public :
   TPaintPage(void){Initialize();};
   TPaintPage(int left, int top, int right, int bottom);
   ~TPaintPage(void){CleanUp();};
   void Initialize(void);
   void CleanUp(void);
   void SetPage(int left, int top, int right, int bottom);
   void SetPageWidth(int left, int right);
   void SetPageHeight(int top, int bottom);
   void ClearPaintPage(TColor Color);
   void AssignFont(TFont* Font);
   Graphics::TBitmap* PaintPage;     // Bitmap for drawing
   TRect* PositionRect;   // position of the paintpage on the plot
   TRect* PageRect;       // rectangle that contains the size of paintpage (like clientrect)
};
//------------------------------------------------------------------------
class TXYPlot : public TPaintBox
{
private:
protected:
   virtual void __fastcall Change();
   TNotifyEvent FOnChange;
   int matherr (struct exception *a);
   virtual void __fastcall SetAxisColor(TColor AColor);
   virtual void __fastcall SetAxisStyle(TAxisStyle AS);
   virtual void __fastcall SetYAxisFormat(TAxisFormat AF);
   virtual void __fastcall SetXAxisFormat(TAxisFormat AF);
   virtual void __fastcall SetYLabelFormat(AnsiString::TStringFloatFormat LF);
   virtual void __fastcall SetXLabelFormat(AnsiString::TStringFloatFormat LF);
   virtual void __fastcall SetYLabelPrecision(int LP);
   virtual void __fastcall SetXLabelPrecision(int LP);
   virtual void __fastcall SetYLabelDigits(int LD);
   virtual void __fastcall SetXLabelDigits(int LD);
   virtual void __fastcall SetShowXLabel(bool XL);
   virtual void __fastcall SetShowYLabel(bool YL);
   virtual void __fastcall SetShowCoordinates(bool CO);
   virtual void __fastcall SetYAutoScale(bool Yb);
   virtual void __fastcall SetXAutoScale(bool Xb);
   virtual void __fastcall SetYMax(double YM);
   virtual void __fastcall SetYMin(double YM);
   virtual void __fastcall SetXMax(double XM);
   virtual void __fastcall SetXMin(double XM);
   virtual void __fastcall SetPointSize(int PS);
   virtual void __fastcall SetShowHorzGrid(bool SG);
   virtual void __fastcall SetShowVertGrid(bool SG);
   virtual void __fastcall SetShowHorzTicks(bool ST);
   virtual void __fastcall SetShowVertTicks(bool ST);
   virtual void __fastcall SetHorzTickLength(int TL);
   virtual void __fastcall SetVertTickLength(int TL);
   virtual void __fastcall SetNumHorzGrid(int NG);
   virtual void __fastcall SetNumVertGrid(int NG);
   virtual void __fastcall SetGridColor(TColor GC);
   virtual void __fastcall SetAutoRedraw(bool AR);
   virtual void __fastcall SetShowXTitle(bool ST);
   virtual void __fastcall SetShowYTitle(bool ST);
   virtual void __fastcall SetShowPlotTitle(bool ST);
   virtual void __fastcall SetXTitle(AnsiString XT);
   virtual void __fastcall SetYTitle(AnsiString YT);
   virtual void __fastcall SetPlotTitle(AnsiString PT);
   virtual void __fastcall SetPlotTitleFont(TFont* TF);
   virtual void __fastcall SetAxisTitleFont(TFont* TF);
   virtual void __fastcall SetAxisLabelFont(TFont* LF);
   virtual void __fastcall SetMouseCoordFont(TFont* CF);

   void Initialize(void);
   void SetProperties(void);
   void CleanUp(void);
   inline void UpdatePaintPage(TPaintPage* Page);
   void FindYMaxMin(void);
   void FindXMaxMin(void);
   void CalculateLogMaxMin(void);
   void CompareMaxMin(void);
   TPlotList* PlotList;
   void RedrawScreen(void);
   void ResizePaintPage(void);
   inline double ScreenX(double X);
   inline double ScreenY(double Y);
   inline double PlotX(double X);
   inline double PlotY(double Y);
   void DrawMouseCoord(void);
   void DrawAxis(void);
   void DrawGrid(void);
   void DrawTicks(void);
   void DrawXAxisLabels(void);
   void DrawYAxisLabels(void);
   void DrawTitles(void);
   inline void DrawPlotsAndLabels(void);
   void DrawAllPlots(void);
   void DrawOnePlot(TPlotListMember* PlotPtr);
   void ClearPlot(void);
   int OldWidth, OldHeight;
   bool FAutoRedraw;
   TColor FAxisColor;                    // color of axis
   TAxisStyle FAxisStyle;                // style of axis
   TAxisFormat FXAxisFormat;             // linear or log axis
   TAxisFormat FYAxisFormat;             // linear or log axis
   bool FShowXLabel;                     // Set to true to show the x labels
   bool FShowYLabel;                     // Set to true to show the y labels
   bool FShowCoordinates;                // Set to true to show mousecoordinates
   bool FYAutoScale;                     // Set to true to scale the y axis automatically
   bool FXAutoScale;                     // Set to true to scale the x axis automatically
   bool FMouseControl;                   // Set to false to prevent user from moving plot with the mouse
   int FPointSize;                       // size of point when PlotStyle = Points
   double FYMax, FYMin;                  // max and min value of y axis
   double FXMax, FXMin;                  // max and min value of x axis
   double FMouseX;
   double FMouseY;
   int FXLabelPrecision, FYLabelPrecision;
   int FXLabelDigits, FYLabelDigits;
   AnsiString::TStringFloatFormat FYLabelFormat;
   AnsiString::TStringFloatFormat FXLabelFormat;
   bool FShowVertGrid, FShowHorzGrid;
   int FNumVertGrid, FNumHorzGrid;
   bool FShowVertTicks, FShowHorzTicks;
   int FVertTickLength, FHorzTickLength;
   TColor FGridColor;
   bool FShowXTitle;
   bool FShowYTitle;
   bool FShowPlotTitle;
   AnsiString FXTitle;
   AnsiString FYTitle;
   AnsiString FPlotTitle;
   TFont* FPlotTitleFont;
   TFont* FAxisTitleFont;
   TFont* FAxisLabelFont;
   TFont* FMouseCoordFont;
   double XLogMin, XLogMax;
   double YLogMin, YLogMax;
   int MX, MY;                           // coordinates of the mouse when dragging and zooming the plot
   int ZoomBoxX, ZoomBoxY;
   bool DragPlot;                        // true if mouse button is pressed
   bool ZoomPlot;                        // true if alt key and mouse button pressed
   bool BoxZoomPlot;
   TPaintPage* PlotPage;                 // pointer to a Bitmap for drawing on.
   TPaintPage* XLabelPage;
   TPaintPage* YLabelPage;
   TPaintPage* XTitlePage;
   TPaintPage* YTitlePage;
   TPaintPage* YTempPage;
   TPaintPage* PlotTitlePage;
   TPaintPage* MouseCoordPage;
   TRect* ZoomBox;

public:
   __fastcall TXYPlot(TComponent* Owner);
   __fastcall TXYPlot(TXYPlot& XYPlot);
   __fastcall ~TXYPlot(void){CleanUp();};
   void AssignPlot(const TXYPlot* XYPlot);
   void AssignPlotData(const TXYPlot* XYPlot);
   void AppendPlotData(const TXYPlot* XYPlot);
   void AssignPlotProperties(const TXYPlot* XYPlot);
   bool XYPlot(double in_array[], int NumPoints, int PNumber=0,
                          TColor PColor = clRed, TPlotStyle PStyle = Lines);
   bool XYPlot(double in_array[], int NumPoints, double minX,
                         double maxX, int PNumber=0, TColor PColor = clRed,
                         TPlotStyle PStyle = Lines);
   bool XYPlot(double xin_array[], double yin_array[],
                          int NumPoints, int PNumber=0,
                          TColor PColor = clRed, TPlotStyle PStyle = Lines);
   bool SetPlotColor(TColor PColor, int PNumber=0);
   bool SetPlotStyle(TPlotStyle PStyle, int PNumber=0);
   bool RemovePlot(int PNumber=0);
   bool GetPlotStyle(TPlotStyle &PStyle, int PNumber=0);
   bool GetPlotColor(TColor &PColor, int PNumber=0);
   bool GetPlotNumPoints(int &NP, int PNumber = 0);
   bool GetPlotX(double x[], int PNumber = 0);
   bool GetPlotY(double y[], int PNumber = 0);
   void SetXYMinMax(double xmin, double xmax, double ymin, double ymax);
   void Redraw(void);

   // new event handlers
   virtual void __fastcall Paint(void);
   virtual void __fastcall MouseMove(Classes::TShiftState Shift, int X, int Y);
   virtual void __fastcall MouseDown(TMouseButton Button,
	TShiftState Shift, int X, int Y);
   virtual void __fastcall MouseUp(TMouseButton Button,
	TShiftState Shift, int X, int Y);
   virtual void __fastcall DblClick(void);
   __property Canvas ;

__published:
   __property ParentFont = {default = true};
   __property Height= {default = 250};
   __property Width= {default = 300};
   __property bool AutoRedraw = {read = FAutoRedraw, write = SetAutoRedraw, default = true};
   __property TColor AxisColor = {read = FAxisColor, write = SetAxisColor, default = clBlack};
   __property TAxisStyle AxisStyle = {read = FAxisStyle, write = SetAxisStyle, default = CenteredBox};
   __property TAxisFormat XAxisFormat = {read = FXAxisFormat, write = SetXAxisFormat, default = Linear};
   __property TAxisFormat YAxisFormat = {read = FYAxisFormat, write = SetYAxisFormat, default = Linear};
   __property bool ShowXLabel = {read = FShowXLabel, write = SetShowXLabel, default = true};
   __property bool ShowYLabel = {read = FShowYLabel, write = SetShowYLabel, default = true};
   __property bool YAutoScale = {read = FYAutoScale, write = SetYAutoScale, default = true};
   __property bool XAutoScale = {read = FXAutoScale, write = SetXAutoScale, default = true};
   __property double YMax = {read = FYMax, write = SetYMax, default = 1};
   __property double YMin = {read = FYMin, write = SetYMin, default = -1};
   __property double XMax = {read = FXMax, write = SetXMax, default = 1};
   __property double XMin = {read = FXMin, write = SetXMin, default = -1};
   __property bool ShowCoordinates = {read = FShowCoordinates, write = SetShowCoordinates, default = true};
   __property int YLabelPrecision = {read = FYLabelPrecision, write = SetYLabelPrecision, default = 4};
   __property int XLabelPrecision = {read = FXLabelPrecision, write = SetXLabelPrecision, default = 4};
   __property int XLabelDigits = {read = FXLabelDigits, write = SetXLabelDigits, default = 4};
   __property int YLabelDigits = {read = FYLabelDigits, write = SetYLabelDigits, default = 4};
   __property AnsiString::TStringFloatFormat YLabelFormat = {read = FYLabelFormat, write = SetYLabelFormat, default = AnsiString::sffGeneral};
   __property AnsiString::TStringFloatFormat XLabelFormat = {read = FXLabelFormat, write = SetXLabelFormat, default = AnsiString::sffGeneral};
   __property int PointSize = {read = FPointSize, write = SetPointSize, default = 2};
   __property bool MouseControl = {read = FMouseControl, write = FMouseControl, default = true};
   __property bool ShowVertGrid = {read = FShowVertGrid, write = SetShowVertGrid, default = true};
   __property bool ShowHorzGrid = {read = FShowHorzGrid, write = SetShowHorzGrid, default = true};
   __property bool ShowVertTicks = {read = FShowVertTicks, write = SetShowVertTicks, default = true};
   __property bool ShowHorzTicks = {read = FShowHorzTicks, write = SetShowHorzTicks, default = true};
   __property int VertTickLength = {read = FVertTickLength, write = SetVertTickLength, default = 6};
   __property int HorzTickLength = {read = FHorzTickLength, write = SetHorzTickLength, default = 6};
   __property int NumVertGrid = {read = FNumVertGrid, write = SetNumVertGrid, default = 3};
   __property int NumHorzGrid = {read = FNumHorzGrid, write = SetNumHorzGrid, default = 3};
   __property TColor GridColor = {read = FGridColor, write = SetGridColor, default = 0x00D8E09E};
   __property double MouseX = {read = FMouseX};
   __property double MouseY = {read = FMouseY};
   __property bool ShowXTitle = {read = FShowXTitle, write = SetShowXTitle, default = true};
   __property bool ShowYTitle = {read = FShowYTitle, write = SetShowYTitle, default = true};
   __property bool ShowPlotTitle = {read = FShowPlotTitle, write = SetShowPlotTitle, default = true};
   __property AnsiString XTitle = {read = FXTitle, write = SetXTitle};
   __property AnsiString YTitle = {read = FYTitle, write = SetYTitle};
   __property AnsiString PlotTitle = {read = FPlotTitle, write = SetPlotTitle};
   __property TFont* PlotTitleFont = {read = FPlotTitleFont, write = SetPlotTitleFont};
   __property TFont* AxisTitleFont = {read = FAxisTitleFont, write = SetAxisTitleFont, nodefault};
   __property TFont* AxisLabelFont = {read = FAxisLabelFont, write = SetAxisLabelFont, nodefault};
   __property TFont* MouseCoordFont = {read = FMouseCoordFont, write = SetMouseCoordFont, nodefault};
   __property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------
#endif
